#ifndef __KERNEL__
#define __KERNEL__
#endif
#ifndef MODULE
#define MODULE
#endif

#include <linux/module.h>
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>

#define SETALLOWED 0x43505500
#define GETALLOWED 0x43505501

static int cpus_allowed_ioctl(struct inode *inode, struct file *filp,
			      unsigned int cmd, unsigned long arg)
{
        switch(cmd) {
	case SETALLOWED:
		if (!arg) arg=1<<current->processor;
		if (!(arg&((1<<smp_num_cpus)-1))) return -EINVAL;
		current->cpus_allowed=arg;
		while (!(current->cpus_allowed&(1<<current->processor))) {
			current->need_resched=1;
			schedule();
		}
		return 0;
	case GETALLOWED:
		{
		int *p=(int*)arg,val;
		if (get_user(val,p)) return -EFAULT;
		if (val<0) return -ESRCH;
		if (val) {
			struct task_struct *p;
			read_lock(&tasklist_lock);
			p=find_task_by_pid(val);
			if (p) val=p->cpus_allowed;
			read_unlock(&tasklist_lock);
			if (!p) return -ESRCH;
		} else
			val=current->cpus_allowed;
		if (put_user(val,p)) return -EFAULT;
		return 0;
		}
	default: return -ENOTTY;
	}
}

static struct file_operations cpus_allowed_fops=
{
	ioctl: cpus_allowed_ioctl
};

static struct miscdevice cpus_allowed_miscdev=
{
	254,
	"cpus_allowed",
	&cpus_allowed_fops
};

int init_module(void)
{
	return misc_register(&cpus_allowed_miscdev);
	return 0;
}

void cleanup_module(void)
{
	misc_deregister(&cpus_allowed_miscdev);
}
